/* Name: holtek_ht1632.c
 * Project: Sure electronics LED board with Holtek HT1632 chip
 * Author: Leonas (Levas) Vabolis
 * Creation Date: 2009-01-18
 * Copyright: (c) 2009 by Savel (aka Levas). http://www.vabolis.lt , http://www.savel.org
 * License: GNU GPL v2
 * Hardware: SAVEL ATMEGA16 testing board, Sure electronics DE-DP016 
 */


#include "holtek_ht1632.h"

// INIT ATMEGA pins/ports
void ledinitPorts(void)				
{
    DDR(LED_PORT) |=_BV(LED_CS) | _BV(LED_WR) | _BV(LED_DATA);
	SET_BIT(LED_CS);
	SET_BIT(LED_WR);
	SET_BIT(LED_DATA);
		
}	

// write single command to HT1632
void writecommand2led(unsigned char address)
{
unsigned char i;
	CLR_BIT(LED_CS);
	SET_BIT(LED_DATA);
	CLR_BIT(LED_WR);
	SET_BIT(LED_WR);
	CLR_BIT(LED_DATA);
	CLR_BIT(LED_WR);
	SET_BIT(LED_WR);
	CLR_BIT(LED_WR);
	SET_BIT(LED_WR);
	for(i=8;i>0;i--)
	{
		if(address & _BV(i-1)) {SET_BIT(LED_DATA);}else{CLR_BIT(LED_DATA);}
		CLR_BIT(LED_WR);
		SET_BIT(LED_WR);
	}
	CLR_BIT(LED_WR);
	SET_BIT(LED_WR);
	SET_BIT(LED_CS);
}


// write adress to HT1632, must be with one of data block.
void writeaddress2led(unsigned char address)
{
unsigned char i;
	CLR_BIT(LED_CS);
	SET_BIT(LED_DATA);
	CLR_BIT(LED_WR);
	SET_BIT(LED_WR);
	CLR_BIT(LED_DATA);
	CLR_BIT(LED_WR);
	SET_BIT(LED_WR);
	SET_BIT(LED_DATA);
	CLR_BIT(LED_WR);
	SET_BIT(LED_WR);
	for(i=7;i>0;i--)
	{
		if(address & _BV(i-1)) {SET_BIT(LED_DATA);}else{CLR_BIT(LED_DATA);}
		CLR_BIT(LED_WR);
		SET_BIT(LED_WR);
	}
}
// write 4 bits of data to HT1632
void writedata2led(unsigned char data)
{
unsigned char i;
for(i=0;i<4;i++)
	{
		if(data & _BV(i)) {SET_BIT(LED_DATA);}else{CLR_BIT(LED_DATA);}
	CLR_BIT(LED_WR);
	SET_BIT(LED_WR);
	}
}
// set CS line to high, to disconnect HT1632, used to switch between address/data and command mode.
#define leaveleds(x) SET_BIT(LED_CS)

// Clear all RAM. Set all LEDs off.
void clearleds(void)
{
char i;
SET_BIT(LED_CS);
CLR_BIT(LED_CS);
writeaddress2led(0);
for(i=0;i<196;i++)
	{ writedata2led(0);	}
SET_BIT(LED_CS);
}

// PWM command 0-15 level 
void setledpwm(unsigned char pwm)
{
SET_BIT(LED_CS);
CLR_BIT(LED_CS);
pwm=pwm & 15;
pwm=pwm | 160;
writecommand2led(pwm);
SET_BIT(LED_CS);
}

// Setup H1632 on sure electronics board
void initleds(void)
{
writecommand2led(CLK_RC);
writecommand2led(44); // PMOS16=44-red. Sure electronics DE-DP016
writecommand2led(BLINK_OFF); 
writecommand2led(SYS_EN);
writecommand2led(LED_ON);
setledpwm(15);
}
