/* Name: main.c
 * Project: Sure electronics LED board with Holtek HT1632 chip and USB control
 * Author: Leonas (Levas) Vabolis
 * Creation Date: 2009-01-18
 * Copyright: (c) 2009 by Savel (aka Levas). http://www.vabolis.lt , http://www.savel.org
 * License: GNU GPL v2
 * Hardware: SAVEL ATMEGA16 testing board, Sure electronics DE-DP016 
 */

#define 	F_CPU 12000000	// Procesoriaus taktinis daznis, reikia delay.h
#include	<avr\io.h>		
#include 	<util\delay.h>	
//---------------------
#include <stdlib.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>  /* sei() */
//#include <avr/eeprom.h>

#include <avr/pgmspace.h>   /* usbdrv.h */
#include "usbdrv/usbdrv.h"
//---------------------
#include "holtek_ht1632.h"
#include "holtek_ht1632.c"
//======================================================================
/* ------------------------------------------------------------------------- */

PROGMEM char usbHidReportDescriptor[22] = {    /* USB report */
    0x06, 0x00, 0xff,              // USAGE_PAGE (Generic Desktop)
    0x09, 0x01,                    // USAGE (Vendor Usage 1)
    0xa1, 0x01,                    // COLLECTION (Application)
    0x15, 0x00,                    //   LOGICAL_MINIMUM (0)
    0x26, 0xff, 0x00,              //   LOGICAL_MAXIMUM (255)
    0x75, 0x08,                    //   REPORT_SIZE (8)
    0x95, 0x80,                    //   REPORT_COUNT (128)
    0x09, 0x00,                    //   USAGE (Undefined)
    0xb2, 0x02, 0x01,              //   FEATURE (Data,Var,Abs,Buf)
    0xc0                           // END_COLLECTION
};


static uchar    currentAddress;
static uchar    bytesRemaining;



/* usbFunctionRead() ,       .
 *     .   usbdrv/usbdrv.h.
 */
uchar   usbFunctionRead(uchar *data, uchar len)
{
uchar i;
for(i=0;i<8;i++){ data[i]=i;}
    if(len > bytesRemaining) len = bytesRemaining;
    currentAddress += len;
    bytesRemaining -= len;
    return len;
}

/* usbFunctionWrite() ,       .
 *     .   usbdrv/usbdrv.h.
 */
uchar   usbFunctionWrite(uchar *data, uchar len)
{
unsigned char i;
    if(bytesRemaining == 0)
        return 1;               /*   */
    if(len > bytesRemaining) len = bytesRemaining;
		writeaddress2led(currentAddress<<1);
		for(i=0;i<len;i++){
			writedata2led(data[i] );
			writedata2led(data[i]>>4);
			}
		leaveleds();
    currentAddress += len;
    bytesRemaining -= len;
    return bytesRemaining == 0; /*  1,      */
}

/* ------------------------------------------------------------------------- */

usbMsgLen_t usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;

    if((rq->bmRequestType & USBRQ_TYPE_MASK) == USBRQ_TYPE_CLASS){    /* HID class */
        if(rq->bRequest == USBRQ_HID_GET_REPORT){  /* wValue: ReportType (highbyte), ReportID (lowbyte) */
            bytesRemaining = 48;
            currentAddress = 0;
            return USB_NO_MSG;  /* use usbFunctionRead() to read */
        }else if(rq->bRequest == USBRQ_HID_SET_REPORT){
            bytesRemaining = 48;
            currentAddress = 0;
			
            return USB_NO_MSG;  /* use usbFunctionWrite() to write */
        }
    }else{
        /* ignore */
    }
    return 0;
}






















// ===========**********************************************================


int main(void)
{
unsigned char i;

ledinitPorts();
initleds();
clearleds();

setledpwm(15);

	wdt_enable(WDTO_1S);
	usbInit();
	usbDeviceDisconnect();
	i = 0;
	while(--i){             /* usb disconnect > 250  */
		wdt_reset();
		_delay_ms(1);
	}
	
    usbDeviceConnect();
    sei();



    for (;;) { 
	        wdt_reset();
			usbPoll();
			
		}
    return 0;
}